import { Buff } from '@cmdcode/buff';
import { get_io_counts } from './util.js';
import { assert, check } from '../util/index.js';
import PSBTSchema from '../schema.js';
import CONST from '../const.js';
export function decode_psbt(psbt_str) {
    const stream = (check.is_hex(psbt_str))
        ? Buff.hex(psbt_str).stream
        : Buff.base64(psbt_str).stream;
    const magic = stream.read(5).hex;
    if (magic !== CONST.PSBT_MAGIC_BYTES) {
        throw new Error('invalid magic for psbt: ' + magic);
    }
    const global = consume_keypairs(stream, CONST.PSBT_GLOBAL_TYPE_MAP);
    const [vin_count, out_count] = get_io_counts(global);
    const inputs = [];
    for (let i = 0; i < vin_count; i++) {
        const pairs = consume_keypairs(stream, CONST.PSBT_INPUT_TYPE_MAP);
        inputs.push(pairs);
    }
    const outputs = [];
    for (let i = 0; i < out_count; i++) {
        const pairs = consume_keypairs(stream, CONST.PSBT_OUTPUT_TYPE_MAP);
        outputs.push(pairs);
    }
    if (stream.size !== 0) {
        throw new Error('stream has excess bytes: ' + new Buff(stream.data).hex);
    }
    return PSBTSchema.data.parse({ global, inputs, outputs });
}
function consume_keypairs(stream, keymap) {
    const pairs = [];
    while (stream.peek(1).num !== 0x00) {
        const kp = decode_keypair(stream, keymap);
        pairs.push(kp);
    }
    stream.read(1);
    return pairs;
}
function decode_keypair(stream, keymap) {
    const klen = stream.read_varint();
    const kdat = stream.read(klen).stream;
    const type = kdat.read_varint();
    let key = (kdat.size !== 0)
        ? kdat.read(kdat.size).hex
        : undefined;
    let label = keymap[type];
    if (type === 0xFC) {
        assert.exists(key);
        const pdat = Buff.bytes(key).stream;
        const plen = pdat.read_varint();
        label = pdat.read(plen).str;
        key = pdat.read(pdat.size).hex;
    }
    const vlen = stream.read_varint();
    const value = stream.read(vlen).hex;
    return { key, label, value };
}
