import { Buff } from '@cmdcode/buff';
import { get_key_type } from './keypair.js';
import { get_io_counts } from './util.js';
import { assert } from '../util/index.js';
import CONST from '../const.js';
export function encode_psbt(data, format = 'hex') {
    const { global, inputs, outputs } = data;
    let [vin_count, out_count] = get_io_counts(global);
    let encoded = CONST.PSBT_MAGIC_BYTES;
    for (const kp of global) {
        encoded += encode_keypair(kp, CONST.PSBT_GLOBAL_TYPE_MAP);
    }
    encoded += '00';
    for (const input of inputs) {
        for (const kp of input) {
            encoded += encode_keypair(kp, CONST.PSBT_INPUT_TYPE_MAP);
        }
        encoded += '00';
        vin_count -= 1;
    }
    if (vin_count > 0) {
        encoded += '00'.repeat(vin_count);
    }
    for (const output of outputs) {
        for (const kp of output) {
            encoded += encode_keypair(kp, CONST.PSBT_OUTPUT_TYPE_MAP);
        }
        encoded += '00';
        out_count -= 1;
    }
    if (out_count > 0) {
        encoded += '00'.repeat(out_count);
    }
    return (format === 'hex')
        ? encoded
        : Buff.hex(encoded).base64;
}
function encode_keypair(keypair, keymap) {
    const { key, label, value } = keypair;
    const type = get_key_type(keymap, label);
    let kb = new Buff(0, 0);
    if (type === 0xFC) {
        assert.exists(key);
        const label_bytes = Buff.str(label);
        const label_vint = Buff.calc_varint(label_bytes.length);
        kb = Buff.join([label_vint, label_bytes, key]);
    }
    else if (key !== undefined) {
        kb = Buff.bytes(key);
    }
    const type_bytes = Buff.num(type);
    const key_bytes = Buff.join([type_bytes, kb]);
    const key_vint = Buff.calc_varint(key_bytes.length);
    const value_bytes = Buff.bytes(value);
    const value_vint = Buff.calc_varint(value_bytes.length);
    return Buff.join([key_vint, key_bytes, value_vint, value_bytes]).hex;
}
