import { Buff } from '@cmdcode/buff';
import { encode_script } from '@scrow/tapscript/script';
import { check } from '../util/index.js';
export function parse_txin(txin) {
    const input = [];
    input.push({ label: 'PREVIOUS_TXID', value: txin.txid });
    input.push({ label: 'OUTPUT_INDEX', value: Buff.num(txin.vout, 4).hex });
    input.push({ label: 'SEQUENCE', value: Buff.num(txin.sequence, 4).hex });
    if (txin.prevout !== undefined) {
        const { value, scriptPubKey } = txin.prevout;
        const prevout = Buff.join([Buff.big(value), encode_script(scriptPubKey, true)]).hex;
        input.push({ label: 'WITNESS_UTXO', value: prevout });
    }
    if (typeof txin.scriptSig !== 'number' && txin.scriptSig.length > 0) {
        const script_sig = encode_script(txin.scriptSig, true).hex;
        input.push({ label: 'FINAL_SCRIPTSIG', value: script_sig });
    }
    if (txin.witness.length > 0) {
        const witness = encode_witness(txin.witness);
        input.push({ label: 'FINAL_SCRIPTWITNESS', value: witness });
    }
    return input;
}
export function parse_txout(txout) {
    const output = [];
    output.push({ label: 'AMOUNT', value: Buff.big(txout.value, 8).hex });
    output.push({ label: 'SCRIPT', value: encode_script(txout.scriptPubKey, true).hex });
    return output;
}
export function decode_witness(bytes) {
    const stream = Buff.bytes(bytes).stream;
    const stack = [];
    const count = stream.read_varint();
    for (let i = 0; i < count; i++) {
        const word = read_data(stream, true);
        stack.push(word ?? '');
    }
    return stack;
}
export function encode_witness(data) {
    const buffer = [];
    if (Array.isArray(data)) {
        const count = Buff.calc_varint(data.length);
        buffer.push(count);
        for (const entry of data) {
            const bytes = (!check.is_empty(entry))
                ? encode_script(entry, true)
                : new Buff(0);
            buffer.push(bytes);
        }
        return Buff.join(buffer).hex;
    }
    else {
        return Buff.bytes(data).hex;
    }
}
function read_data(stream, varint) {
    const size = (varint === true)
        ? stream.read_varint('le')
        : stream.size;
    return size > 0
        ? stream.read(size).hex
        : null;
}
