import { TxData }     from '@scrow/tapscript'
import { parse_txin, parse_txout } from './tx.js'

import { encode_tx, parse_tx } from '@scrow/tapscript/tx'

import {
  PSBTData,
  PSBTGlobalRecord,
  PSBTInputSet,
  PSBTOutputSet
} from '@/types.js'

import PSBTSchema from '@/schema.js'

export function create_psbt (tx : string | TxData) : PSBTData {
  const global : PSBTGlobalRecord[] = [],
        inputs : PSBTInputSet[]     = [],
        outputs : PSBTOutputSet[]   = []

  const txdata = parse_tx(tx)
  const txbase = encode_tx(txdata, true).hex

  for (const txin of txdata.vin) {
    inputs.push(parse_txin(txin))
  }

  for (const txout of txdata.vout) {
    outputs.push(parse_txout(txout))
  }

  global.push({ label: 'UNSIGNED_TX', value: txbase })

  // We also need to check for required included, required excluded, and allowed.
  return PSBTSchema.data.parse({ global, inputs, outputs }) as PSBTData
}
