import { TxPrevout }      from '@scrow/tapscript'
import { normalize_psbt } from './parse.js'
import { parse_txin }     from './tx.js'
import { assert }         from '@/util/index.js'

import {
  PSBTData,
  PSBTemplate
} from '@/types.js'

import PSBTSchema from '@/schema.js'

export function update_psbt (
  psbt  : string | PSBTData,
  templ : PSBTemplate
) : PSBTData {
  const data = normalize_psbt(psbt)
  // Unpack PSBT data object.
  const { global, inputs, outputs } = data
  // Push elements from template into PSBT data.
  global.push(...templ.global  ?? [])
  inputs.push(...templ.inputs  ?? [])
  outputs.push(...templ.outputs ?? [])
  // Validate and return the new PSBT.
  return PSBTSchema.data.parse({ global, inputs, outputs }) as PSBTData
}

export function add_psbt_utxo (
  psbt : string | PSBTData,
  utxo : TxPrevout,
  index = 0
) {
  const data = normalize_psbt(psbt)
  // Unpack PSBT data object.
  const { global, inputs, outputs } = data
  // Assert that the utxo index does not exceed the PSBT input index.
  assert.ok(data.inputs.length > index, 'index exceeds psbt input count')
  // Add utxo records to PSBT input.
  inputs[index] = parse_txin(utxo)
  // Validate and return the new PSBT.
  return PSBTSchema.data.parse({ global, inputs, outputs }) as PSBTData
}
